REFUNDED_CAP_COL = 'refunded_cap'
INTERSET_COL = 'interset'
CAP_COL = 'cap'
YEAR_COL = 'annees'
MONTHLY_COL = 'mensualites'
INS_T_COL = 'assur Tom'
INS_M_COL = 'assur Mel'
INS_ALL_COL = 'assur tot'
MONTHLY_INS_COL = 'mensu avc assur'

ALL_COLS = [YEAR_COL, MONTHLY_COL, INS_ALL_COL, MONTHLY_INS_COL, REFUNDED_CAP_COL, INTERSET_COL, CAP_COL]

INS_T_ACT_COL = 'assur Tom reel'
INS_M_ACT_COL = 'assur Mel reel'
INS_ALL_ACT_COL = 'assur tot reel'

COSTS_COL = 'costs'
INS_RATES_COL = 'ins_rates'



import numpy as np
import pandas as pd

from immo_tools.constants import *


class UnitException(Exception):
    pass


class Calculator:
    def __init__(self):
        self.summary = None

    def get_cost(self, sales_year=8, to_string=False):
        if self.summary is None:
            return None
        cost = self.summary \
                   .groupby(by=[YEAR_COL]) \
                   .sum() \
                   .iloc[:sales_year, :] \
                   .loc[:, [INTERSET_COL, INS_ALL_COL, REFUNDED_CAP_COL]] \
            .sum() \
            .round(2)
        if to_string:
            return cost.to_markdown()
        else:
            return cost


def convert_rate_per_month(rate, n_per=12):
    return (np.power(1 + rate, 1 / n_per)) - 1


def convert_duration_in_years(dur, unit='month'):
    if unit == 'month':
        return int(dur / 12)
    elif unit == 'year':
        return int(dur)
    raise UnitException(f'unité {unit} non gérée')


class Loan(Calculator):
    summary = None

    def __init__(self, duration, amount, year_rate, insurance_rate, duration_unit='month', build_summary=False,
                 ins_on_rest=False):
        super().__init__()
        self.amount = amount
        self.insurance_rate = insurance_rate / 100
        self.duration = convert_duration_in_years(duration, duration_unit)
        self.year_rate = year_rate / 100
        self.month_rate = convert_rate_per_month(self.year_rate)
        self.monthly = self.get_monthly()
        self.ins_on_rest = ins_on_rest
        if build_summary:
            self.build_summary(ins_on_rest)

    def get_monthly(self):
        if self.month_rate == 0:
            return self.amount / self.duration / 12
        m = np.power(1 + self.month_rate, self.duration * 12)
        a = self.amount * self.month_rate * m
        b = m - 1
        return a / b

    def build_summary(self, ins_on_rest=False):
        self.summary = self.get_summary(ins_on_rest)

    def get_summary(self, ins_on_rest=False):
        summary = []
        cap = int(self.amount)
        year = 0
        for idx, period in enumerate(range(self.duration * 12)):
            if idx % 12 == 0:
                year += 1
            interest = self.month_rate * cap
            refunded_cap = self.monthly - interest
            cap = cap - refunded_cap

            a = cap if ins_on_rest else self.amount
            ins = a * (self.insurance_rate / 12)

            summary.append({
                YEAR_COL: int(year),
                MONTHLY_COL: self.monthly,
                INS_ALL_COL: ins,
                MONTHLY_INS_COL: self.monthly + ins,
                REFUNDED_CAP_COL: refunded_cap,
                INTERSET_COL: interest,
                CAP_COL: cap
            })
        return pd.DataFrame(summary)

    def get_interests(self):
        if self.summary is None:
            return None
        return self.summary.loc[:, [INTERSET_COL]].sum().round(2).values[0]


def build_loan(duration, amount, year_rate, insurance_rate, duration_unit='month', build_summary=False,
               ins_on_rest=False):
    return Loan(duration, amount, year_rate, insurance_rate, duration_unit, build_summary,
                ins_on_rest)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['immo_tools']

package_data = \
{'': ['*']}

install_requires = \
['numpy>=1.18.1,<2.0.0', 'pandas>=1.0.1,<2.0.0', 'tabulate>=0.8.6,<0.9.0']

setup_kwargs = {
    'name': 'immo-tools-lib',
    'version': '0.1.3',
    'description': 'calcul des couts de credit immobilier',
    'long_description': "# immo tools lib\n\n## installation\n\navec pip :\n````shell script\npip install immo-tools-lib\n````\n\navec poetry :\n````shell script\npoetry add immo-tools-lib\n````\n\n## usage\n\n````python\nfrom immo_tools import calculator\n\nduration = 300\namount = 250000\nyear_rate = 1.5\ninsurance_rate = .26\n\nloan = calculator.build_loan(\n    duration, \n    amount, \n    year_rate, \n    insurance_rate, \n    build_summary=True, \n    duration_unit='month')\n\n# Tableau d'amoritssement :\nloan.summary\n\n# montant total des intérêts payés :\nloan.get_interests()\n\n# mensualités sans assurance :\nloan.get_monthly()\n\n# Coût de l'emprunt au bout de 10 ans :\nloan.get_cost(10)\n````\n\n## développement\n\n### tests\n\nlancer les tests :\n\n````shell script\npytest\n````\n\navec le coverage :\n\n````shell script\npytest --cov\n````\n\nlancer les tests avec nox : (attention, c'est super long !):\n\n````shell scripts\nnox -r\n````\n\nPermet de lancer les tests sur plusieurs versions de python et de manière isollée.\nNox crée son propre environnement virtuel et y install les dépendances.\nL'option ``-r`` permet déviter de recréer tout l'environnement virtuel à chaque fois.\n\n### build & publish\n\nbuild :\n\n````shell script\npoetry build\n````\n\n\npublish sur pypi :\n\n````shell script\npoetry publish --build\n````\n\n",
    'author': 'thomas.marquis.dev',
    'author_email': 'thomas.marquis314@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/thomas-marquis/immo-tools-lib',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
